/* 
 * E-LogMonitor v1.0  (C) 2000 Bart Cichosz (barrrtdev@usa.net)
 *
 * This program is an enlightenment epplet to monitor
 * various log files for occurence of specified
 * keywords.
 *
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc.
 * 59 Temple Place - Suite 330
 * Boston, MA  02111-1307, USA
 *
 */



#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <epplet.h>

#define CONFIG_FILENAME	".E-LogMonitor.conf"
#define LOG_FILENAME	".E-LogMonitor.log"


/*#define	NO_OF_KEYWORDS	18 */
#define KEY_MAX_ROW		4			
#define KEY_MAX_COL		5

/*#define BUF_SIZE		1024 */
#define FN_SIZE			1024
#define MAX_KEYWORD_LEN	256


Epplet_gadget		EppMenu;								
Epplet_gadget		EppImg[KEY_MAX_ROW][KEY_MAX_COL];
long				EppImgData[KEY_MAX_ROW][KEY_MAX_COL];

int					DefCon[4]={3,15,60,600};
int					BtnFlagShowLog;				/* state of Show mode button */
int					BtnFlagReset;				/* state of Reset mode button */
int					BtnFlagInfo;				/* state of Info mode button */
						
char				Img[6][512];				/* full path+fn to button images */
char				ConfigFile[512];			/* full path+fn to config file */
char				RunWhenAlert[512];			/* command to run when keyword found */
char				LogFile[512];				/* full path+fn to output log file */ 
FILE				*LFile;						/* pointer to output log file */

/* keywords array will contain info about each keyword */
typedef struct {
	int				inuse;				/* 1 if in use, or 0*/
	char			keyword[MAX_KEYWORD_LEN];
	char			filename[FN_SIZE];
	int				fd;					/* file descriptor */
	int				x,y;				/* location on epplet */
	time_t			last;
	int				level;
} KEYWORD;

KEYWORD keywords[KEY_MAX_ROW][KEY_MAX_COL];



/* file descriptors */
typedef struct {
	int				isopen;
	char			filename[FN_SIZE];
	FILE			*f;
} FILEDESC;

FILEDESC 	*fds;


int			fdslen;								/* number of elements in fds */
long		sizeoffds;							/* sizeof(FILEDESC) */



/**************************************************
 *
 * like perl's chop, more or less
 *
 **************************************************/

void
bsl_chop(char *s) {
	
	char	*q;
	q=s+strlen(s);
	
	while(q>=s && ((*q)==0 || (*q)==0x0A || (*q)==0x0D))
		*(q--)='\0';
}


/**************************************************
 *
 * cleanup after yourself
 *
 **************************************************/

static void
cb_close(void *data) {

	FILEDESC	*fdstemp;
	int			i;

	/* epplet cleanup */
	Esync();
	Epplet_cleanup();
	data=NULL;
	
	/* close log files */
	if(fds!=NULL) {
		fdstemp=fds;
		for(i=0; i<fdslen;i++) {
			if(fdstemp->f!=NULL) fclose(fdstemp->f);
			fdstemp++;
		}
	}

	/* Close output log file */
	if(LFile!=NULL) {
		fclose(LFile);
	}

	
	/* free fds memory */
	if(fds!=NULL)  free(fds);
	
	exit(0);
}	

/**************************************************
 *
 * check if filename is already in fds, if not
 * increase size of memory allocated to fds and
 * append new filename. Either way return the 
 * index of where the filename is stored
 *
 **************************************************/

int
fdsadd(const char *filename) {
	
	int 		i;
	FILEDESC	*fdstemp;
		
	if(fds==NULL) {
		/* First on - easy job */
		if((fds=(FILEDESC *)malloc(sizeoffds))==NULL) {
			printf("Cannot allocate memory for fds.\n");
			cb_close(NULL);
		}
		
		fds->isopen=0;
		fds->f=NULL;
		strncpy(fds->filename,filename,FN_SIZE);
		fdslen=1;
		return(0);
	} else {
		/* try finding a fd with same filename, then return its id */
		for(i=0; i<fdslen;i++) {
			fdstemp=fds+i;
			if(strcmp((char *)&fdstemp->filename,filename)==0) {
				/* found one */
				return(i);
			}
		}

		/* did not find this filename, so it's a new file */

		if((fds=(FILEDESC *)realloc(fds,(1+fdslen)*sizeoffds))==NULL) {
			printf("Cannot allocate memory for fds.\n");
			cb_close(NULL);
		}
		
		fdstemp=fds+fdslen;
		fds->f=NULL;
		fdstemp->isopen=0;
		strncpy((char *)&fdstemp->filename,filename,FN_SIZE);
		fdslen++;
		return(fdslen-1);
	}
}			

				
/**************************************************
 *
 * read configuration file
 *
 **************************************************/

int
sla_readconfig(void) {

	int 	row,col;
	FILE	*conf;
	char	*str, *substr;
	char	*fbuf;	
	int		fbuf_size;
	ssize_t	n;
	
	/* zero out contents of keywords array */
	for(row=0; row<KEY_MAX_ROW; row++) 
		for(col=0;col<KEY_MAX_COL; col++) 
			bzero(&keywords[row][col],(size_t)sizeof(KEYWORD));

	/* load configuration */
	if(!(conf=fopen(ConfigFile,"r"))) {
		printf("Cannot open configuration file: %s\n\n",ConfigFile);
		cb_close(NULL);
	}

	fbuf_size=255;
	if((fbuf=(char *)malloc(fbuf_size))==NULL) {
		printf("Cannot allocate %i bytes.\n",fbuf_size);
		exit(0);
	}
	while((n=getline(&fbuf,&fbuf_size,conf)) >= 0) {
		/*printf("%i ",n);
		printf("Read config line: %s",fbuf); */
		if(n>0) {
			switch(*fbuf) {
				case '@' :			/* alarm setting */
					str=fbuf+1;		/* start after '@' */
					/* row */
					if((substr=strsep(&str,":"))==0) {
						printf("Row not found.\n\n");
						printf("Error in configuration file. ");
						return(0);
					}
					row=atoi(substr);
	
					if((substr=strsep(&str,":"))==0) {
						printf("Error in configuration file. ");
						printf("Column not found.\n\n");
						return(0);
					}
					col=atoi(substr);
	
					if((substr=strsep(&str,":"))==0) {
						printf("Error in configuration file. ");
						printf("Filename not found.\n\n");
						return(0);
					}

					strncpy(keywords[row][col].filename,substr,strlen(substr));
					bsl_chop(keywords[row][col].filename);
					
					substr=substr+1+strlen(substr);					
					strncpy(keywords[row][col].keyword,substr,strlen(substr));
	
					bsl_chop(keywords[row][col].keyword);
					
					
					keywords[row][col].inuse=1;
					break;
				case '!':
					str=fbuf+1;
					strncpy(RunWhenAlert,str,511);
					break;

					
				default:
					/* Ignore any other lines for now */
			}
		}
	}

					
	free(fbuf);	
	fclose(conf);
	

	
	/* create list of fds */
	for(row=0; row<KEY_MAX_ROW; row++) {
		for(col=0; col<KEY_MAX_COL; col++) {
			if(keywords[row][col].inuse) {
				keywords[row][col].fd=fdsadd(keywords[row][col].filename);
			}
		}
	}

	return(1);
}


/**************************************************
 *
 * check each file in fds for keywords in keywords
 *
 **************************************************/

int
checklogs() {

	int			row, col,n;
	int			fdsi;	
	FILEDESC	*fdstemp;
	char		*fbuf;
	int			fbuf_size;
	int			Alert;
	
	
	Alert=0;
	
	/* just in case */
	if(fds==NULL) return (0);
	
	/* Allocate initial buffer */
	if((fbuf=(char *)malloc(255))==NULL) {
		printf("Cannot allocate memory (255 bytes).\n");
		cb_close(NULL);
	} 
	fbuf_size=254;

	for(fdsi=0; fdsi<fdslen; fdsi++) {
		fdstemp=fds+fdsi;
		/* only if the file is open */
		if(fdstemp->f!=NULL) {
			while((n=getline(&fbuf,&fbuf_size,fdstemp->f))>0) {
				/* check each keyword if matches */
				bsl_chop(fbuf);
				for(row=0;row<KEY_MAX_ROW;row++) {
					for(col=0;col<KEY_MAX_COL;col++) {
						/* only if inuse flag is set */
						if(keywords[row][col].inuse) {
							if(strstr(fbuf,keywords[row][col].keyword)!=NULL) {
								/* Found a keyword */
								if(LFile!=NULL) {
									fprintf(LFile,"(%s)(%s) %s.\n",keywords[row][col].keyword,fdstemp->filename,fbuf);
									fflush(LFile);
									Alert=1;
								}
								keywords[row][col].last=time(NULL);
							}
						}
					}
				}
			}
		}
	}


	free(fbuf);

	if(Alert && RunWhenAlert[0]!='\0')
		Epplet_spawn_command(RunWhenAlert);

	
	return(0);	

}


/**************************************************
 *
 * if needed, change button images
 *
 **************************************************/


void paintbuttons(void) {


	int 		row, col;
	time_t		t;
	int			dt;
	int			newDefCon;
	
	t=time(NULL);
	for(row=0;row<KEY_MAX_ROW; row++) {
		for(col=0; col<KEY_MAX_COL; col++) {
			if(keywords[row][col].inuse) {
				dt=t-keywords[row][col].last;
				
				if(dt<=DefCon[0]) {
					newDefCon=0;
				} else if(dt<=DefCon[1]) {
					newDefCon=1;
				} else if(dt<=DefCon[2]) {
					newDefCon=2;
				} else if(dt<=DefCon[3]) {
					newDefCon=3;
				} else {
					newDefCon=4;
				}

				if(newDefCon!=keywords[row][col].level) {
					/* change of level */
					keywords[row][col].level=newDefCon;
					/*printf("change to %i\n",newDefCon); */
					
					
					Epplet_change_button_image(EppImg[row][col],&Img[newDefCon][0]);
					Esync();		
				} 
			} else {
				/* not in use */
				if(keywords[row][col].level<99) {
					
					/* just changed from level_x */
					Epplet_change_button_image(EppImg[row][col],&Img[5][0]);
					Esync();
					keywords[row][col].level=99;
				}
			}
			
		}
	}
}
	
/**************************************************
 *
 * Called by timer, check logs, paint buttons,
 * setup next timer call
 *
 **************************************************/

static void
cb_timer(void *data) {
	
	checklogs();
	
	paintbuttons();


	
	Epplet_timer(cb_timer,NULL,1,"TIMER");
	data=NULL;
}

/**************************************************
 *
 * Show output log
 *
 **************************************************/

void
cb_ShowLog(void *d) {

	char *buffer;

		
	if((buffer=(char *)malloc(1024))) {
		strcpy(buffer,"Eterm -e tail -f ");
		strcat(buffer,LogFile);
		Epplet_spawn_command(buffer);
		free(buffer);
	}
}



/**************************************************
 *
 * Called when an alarm button is clicked, act
 * depending on flags
 *
 **************************************************/

void
cb_EppImgClicked(void *d) {
	
	int		row,col;
	char	*buffer;
	
	
	row=(*((long*)d))/100;
	col=(*((long*)d))-(100*row);	
	
	if(keywords[row][col].inuse) {

		if(BtnFlagReset) 
			keywords[row][col].last=0;
	

		
	
		if(BtnFlagShowLog) {
			if((buffer=(char *)malloc(1024))) {
				strcpy(buffer,"Eterm -e tail -f ");
				strcat(buffer,keywords[row][col].filename);
				Epplet_spawn_command(buffer);
				free(buffer);
			}
		}

		if(BtnFlagInfo) {

			if((buffer=(char *)malloc(1024))) {
				strcpy(buffer,"This button monitors file ");
				strcat(buffer,keywords[row][col].filename);
				strcat(buffer,"\nfor occurence of keyword\n");
				strcat(buffer,keywords[row][col].keyword);

				Epplet_dialog_ok(buffer);
			
			
				free(buffer);
			}
		}
		
	
		paintbuttons();	
	
	}

	
}
		
/**************************************************
 *
 * Display ABOUT box
 *
 **************************************************/


void
cb_about(void *data) {

	char	*buffer;

	if((buffer=(char *)malloc(1024))) {
		strcpy(buffer, "E-LogMonitor v1.0\n\n(C) 2000 Bart Cichosz (barrrtdev@usa.net)\n\n\n\n");
		strcat(buffer,"Help Mother Earth!\nSupport environment friendly companies!\n\n"); 
						
		strcat(buffer,"Aquatic Cellulose International (OTC:BB AQCI)\n");
		strcat(buffer,"http://www.aquaticcellulose.com/\n");
		strcat(buffer,"http://www.geocities.com/club_aqci_2/Club_AQCI_2_home.html\n");
		strcat(buffer,"http://www.ragingbull.com/mboard/boards.pl?board=AQCI\n\n");
		
		strcat(buffer,"(The above is NOT meant as investment advice.\nALWAYS do your own DD.)\n\n");
		Epplet_dialog_ok(buffer);
			
		free(buffer);
	}
}



/**************************************************
 *
 * DUH!
 *
 **************************************************/



int 
main(int argc, char **argv) {
	
	int 		i,row,col;	
	FILEDESC	*fdstemp;
	
	char		bufs[1024];
	int			noConfigFile;
	
	fds=NULL;
	fdslen=0;
	sizeoffds=sizeof(FILEDESC);
	BtnFlagReset=0;
	BtnFlagShowLog=0;	
	BtnFlagInfo=1;
	noConfigFile=0;
	RunWhenAlert[0]='\0';	

	/* Initialize Img array - containing paths to images */
	for(i=0; i<6; i++) {
		strcpy(&Img[i][0],getenv("EROOT"));
		strcat(&Img[i][0],"/epplet_data/E-LogMonitor-");
		if(i<5) {
			sprintf(bufs,"L%i",i);
		} else {
			sprintf(bufs,"Off");
		}
		strcat(&Img[i][0],bufs);
		strcat(&Img[i][0],".png");
	}

	/* Check if configuration file exists, if it does not,
	 * create one and display a messaege */

	strcpy(ConfigFile,getenv("HOME"));
	strcat(ConfigFile,"/");
	strcat(ConfigFile,CONFIG_FILENAME);
	
	if(access(ConfigFile,F_OK)!=0) {
		strcpy(bufs,"cp ");
		strcat(bufs,getenv("EROOT"));
		strcat(bufs,"/epplet_data/E-LogMonitor.conf ");
		strcat(bufs,ConfigFile);
		system(bufs);	
		noConfigFile=1;
	}

	strcpy(LogFile,getenv("HOME"));
	strcat(LogFile,"/");
	strcat(LogFile,LOG_FILENAME);
	
	/* read the configuration file */	
	sla_readconfig();

	/* open log files */
	if(fds==NULL) {
		printf("No files to monitor were selected");
		cb_close(NULL);
	}
	fdstemp=fds;
	for(i=0; i<fdslen; i++) {
		
		if((fdstemp->f=fopen(fdstemp->filename,"r"))==NULL) {
			printf("Cannot open %s. This file will not be monitored.\n",fdstemp->filename);
		} else {
			fdstemp->isopen=1;
			fseek(fdstemp->f,0,SEEK_END);
		}	
	
		fdstemp++;
	}


	/* Open output log */
	LFile=fopen(LogFile,"a");	


	
	/* Epplet initialization code */

	Epplet_Init("E-LogMonitor","0.1","Enlightenment Log Monitor Epplet",3,3,argc,argv,0);
	for(row=0; row<KEY_MAX_ROW;row++) {
		for(col=0;col<KEY_MAX_COL;col++) {
			EppImgData[row][col]=(100*row)+col;
			Epplet_gadget_show(EppImg[row][col]=Epplet_create_button(NULL,&Img[5][0],6+(col*8),2+(row*8),6,6,NULL,0, 0,cb_EppImgClicked,&EppImgData[row][col]));
		}
	}	
	
		
		
		
		
	Epplet_gadget_show(Epplet_create_togglebutton("i",NULL,17,35,9,12,&BtnFlagInfo,NULL,NULL));
	Epplet_gadget_show(Epplet_create_togglebutton("r",NULL,27,35,9,12,&BtnFlagReset,NULL,NULL));
	Epplet_gadget_show(Epplet_create_togglebutton("s",NULL,37,35,9,12,&BtnFlagShowLog,NULL,NULL));
	EppMenu=Epplet_create_popup();
	
	Epplet_add_popup_entry(EppMenu,"exit",NULL,cb_close,NULL);
	Epplet_add_popup_entry(EppMenu,"about",NULL,cb_about,NULL);
	Epplet_add_popup_entry(EppMenu,"show log",NULL,cb_ShowLog,NULL);
	Epplet_gadget_show(Epplet_create_popupbutton(NULL,NULL,2,35,12,12,"ARROW_UP",EppMenu));

	
	Epplet_show();

	if(noConfigFile) {
		strcpy(bufs,"E-LogMonitor: Configuration file not found.\n\n");
		strcat(bufs,"A sample configuration file (");
		strcat(bufs,ConfigFile);
		strcat(bufs,")\n\nhas been created. Please edit it to your\n\n");
		strcat(bufs,"liking and restart E-LogMonitor.\n\n");
		Epplet_dialog_ok(bufs); 
	}
		
	paintbuttons();
	Epplet_timer(cb_timer,NULL,1,"TIMER");
	Epplet_Loop();
	

	return(0);

}



	

	
	
